@@ -28,7 +28,7 @@
 static const char copyright[] = "\
 @(#)Copyright (C) @BABOLO  2002 Dec 17\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: cmd_copy.c,v 1.32 2009/10/27 02:01:14 babolo Exp $";
+static const char rcsid[] = "$Id: cmd_copy.c,v 1.34 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -37,10 +37,10 @@ static const char rcsid[] = "$Id: cmd_copy.c,v 1.32 2009/10/27 02:01:14 babolo E
 #include <limits.h>
 #include <string.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -171,7 +171,7 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_r *r) {
                 ifBLIN_QV1(exenv->options->flags) warn("pgoblin_copyin: mife_read");
                 e = -1;
             }
-            ifBLIN_QV5(exenv->options->flags) fprintf(stderr, "mife_read=%d,", BLIN_I(f));
+            ifBLIN_QV5(exenv->options->flags) fprintf(stderr, "mife_read=%d,", (int)BLIN_I(f));
             ofbase = mife_get(IO_IN->mife, IO_IN->offset);
             if  (!e) {
                 for (q = ofbase; q < (ofbase + f) && k != finde_end; k = automa[k][fich[(u_char)*q++]])
@@ -179,9 +179,9 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_r *r) {
                 if  (q - ofbase) {
                     e = DBASE(R_CONN->flags)->putcopy(R_CONN, ofbase, q - ofbase) < 1;
                     ifBLIN_QV4(exenv->options->flags)
-                        fprintf(stderr, "putcopy=%d %d (%d)\n", e, k, BLIN_I(q - ofbase));
+                        fprintf(stderr, "putcopy=%d %d (%d)\n", e, k, (int)BLIN_I(q - ofbase));
                 } else ifBLIN_QV4(exenv->options->flags) {
-                    fprintf(stderr, "no putcopy %d (%d)\n", k, BLIN_I(q - ofbase));
+                    fprintf(stderr, "no putcopy %d (%d)\n", k, (int)BLIN_I(q - ofbase));
         }   }   }
         if  (!e && k != finde_end) {                       /*  ()    */
             const char *enstin = "\n\\.\n";
