@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2004 Nov 18
+ * Copyright (C)2004..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2004 Nov 18\n\
+@(#)Copyright (C)2004..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: cmd_file.c,v 1.41 2009/11/17 09:04:40 babolo Exp $";
+static const char rcsid[] = "$Id: cmd_file.c,v 1.45 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -42,12 +42,12 @@ static const char rcsid[] = "$Id: cmd_file.c,v 1.41 2009/11/17 09:04:40 babolo E
 #include <limits.h>
 #include <errno.h>
 #include <stdio.h>
-#include <getCGIparm.h>
-#include <mife.h>
 #include <fts.h>
 #include <err.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <getCGIparm.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -311,7 +311,7 @@ pgoblin_getstr(pgoblin_exenv *exenv, pgoblin_r *r) {
     if  (!(fi = funopen( (void*)(IO_OUT->onu)
                        , NULL, (int (*)(void*, const char*, int))mife_writ, NULL, NULL
         ) )            ) {
-        ifBLIN_QV1(exenv->options->flags) warn("funopen #0 on %d", r[PGO_COUT].r);
+        ifBLIN_QV1(exenv->options->flags) warn("getstr funopen #0 on %d", r[PGO_COUT].r);
         ex = EX_OSERR;
     } else {
         for (j = 0, l = 1; !(IO_IN->mife->flag & MIFE_EOFL); IO_IN->offset += bl) {
@@ -334,7 +334,7 @@ pgoblin_getstr(pgoblin_exenv *exenv, pgoblin_r *r) {
                     default:   if (l) fprintf(fi, "%d\t", j); l = 0; fprintf(fi, "%c", *cont); break;
         }   }   }
         if  (!l) fprintf(fi, "\n");
-        fprintf(fi, "\\.\n");
+        if  (~exenv->options->flags & PGOBLIN_COPEND_NOGEN) fprintf(fi, "\\.\n");
     }
     if  (fclose(fi) < 0) {
         ifBLIN_QV1(exenv->options->flags) warn("fclose #0 on %d", r[PGO_COUT].r);
@@ -344,6 +344,192 @@ pgoblin_getstr(pgoblin_exenv *exenv, pgoblin_r *r) {
     return(ex);
 }
 
+static const char *statenames[] =
+{ "bg", "cp", "be", "cq", "qq", "xx"};
+
+enum states
+{ bg /*                                      */
+, cp /*                  */
+, be /*                        */
+, cq /*                    */
+, qq /*                         */
+, xx
+};
+
+static const char *pn[] = 
+{ "- ", "- ", "- ", "- ", "- ", "- ", "- ", "- "
+, "- ", "Ct", "Cd", "Cb", "Cp", "Cn", "- ", "Cq"
+, "- ", "- ", "- ", "- ", "Cx", "- ", "- ", "- "
+};
+
+#define Ct  0x400000 /* \t                                 */
+#define Cd  0x200000 /*            */
+#define Cb  0x100000 /*             */
+#define Cp  0x080000 /*      */
+#define Cn  0x040000 /*                     */
+#define Cq  0x010000 /*   ++                */
+#define Cx  0x000800 /*                           */
+#define C_state 0xFF /*                  */
+
+static const char *clasnames[] =
+{ "L0", "Ll", "Lb", "Ls", "Lq", "Lc", "Lo", "Lx"};
+
+#define L0 0 /* \0                                         */
+#define Ll 1 /*                                 */
+#define Lb 2 /*                                      */
+#define Ls 3 /*                                */
+#define Lq 4 /* "                                          */
+#define Lc 5 /* ,                                          */
+#define Lo 6 /*                                   */
+#define Lx 7
+
+static const char clasi[257] =
+{ L0
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lb, Ll, Ls,  Lo, Ls, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lb, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
+};
+
+static u_int32_t automa[xx][Lx] =
+/*    \0 L0     \n Ll  \b Lb x Ls         " Lq      ; Lc         * Lo  *    */
+{{       xx, Cn|Cb|bg,    bg,  bg,          cq, Ct|Cb|bg,       Cp|cp}/* bg */
+,{ Cn|Cb|xx, Cn|Cb|bg, Cq|be,  cp,          cp, Ct|Cb|bg,       Cp|cp}/* cp */
+,{ Cn|Cb|xx, Cn|Cb|bg,    be,  be, Cd|Cb|Cp|cp, Ct|Cb|bg, Cd|Cb|Cp|cp}/* be */
+,{    Cx|xx,    Cp|cq, Cp|cq,  cq,          qq,    Cp|cq,       Cp|cq}/* cq */
+,{ Cn|Cb|xx, Cn|Cb|bg,    be,  qq,       Cp|cq, Ct|Cb|bg,          cp}/* qq */
+};
+
+int
+pgoblin_getcsv(pgoblin_exenv *exenv, pgoblin_r *r) {
+    int ex = EX_OK, n, max = 0;
+    u_char c, q, *t;
+    char class[257];
+    FILE *fi;
+
+    ifBLIN_QV4(exenv->options->flags) {
+        fprintf( stderr, "+pgoblin_getcsv %d %d %d %d"
+               , r[PGO_COUT].r, r[PGO_CJOB].r, r[PGO_CCON].r, r[PGO_CIN].r
+               );
+        if  (IO_CTL->text) fprintf(stderr, " =%s~", (char*)(IO_CTL->text));
+        fprintf(stderr, "\n");
+    }
+    c = ';';
+    q = '\"';
+    if  ((t = IO_CTL->text)) {
+        c = t[0];
+        q = t[1];
+        errno = 0;
+        if  (t[2]) max = strtol((char*)&(t[2]), NULL, 0);
+        if  (errno) {
+            ifBLIN_QV1(exenv->options->flags) warn("getcsv strtol=%s~", (char *)IO_CTL->text);
+            ERROUT(EX_DATAERR, EINVAL);
+        }
+        if  (max < 0) {
+            ifBLIN_QV1(exenv->options->flags) warnx("getcsv strtol %d<0", max);
+            ERROUT(EX_DATAERR, EINVAL);
+    }   }
+    bcopy(clasi, class, 257);
+    class[c + 1] = Lc;
+    class[q + 1] = Lq;
+    if  (!(fi = funopen( (void*)(IO_OUT->onu)
+                       , NULL, (int (*)(void*, const char*, int))mife_writ, NULL, NULL
+        ) )            ) {
+        ifBLIN_QV1(exenv->options->flags) warn("getcsv funopen #0 on %d", r[PGO_COUT].r);
+        ex = EX_OSERR;
+    } else {
+        enum states state;
+        u_int32_t control;
+        char *cont;
+        ssize_t l;
+        u_char p;
+        int s, rs;
+
+        ifBLIN_QV5(exenv->options->flags) BLIN_STATEHEADER(stderr, pn)
+        for (state = bg, control = 0, n = 0, rs = 0
+            ; state < xx
+            ; IO_IN->offset++, state = control & C_state
+            ) {
+            if  ((l = mife_read(IO_IN->mife, 1, IO_IN->offset)) < 0) {
+                ifBLIN_QV1(exenv->options->flags) warn("getcsv mife_read");
+                ex = EX_IOERR;
+                break;
+            }
+            if  (!(cont = mife_get(IO_IN->mife, IO_IN->offset))) {
+                ifBLIN_QV1(exenv->options->flags) warn("getcsv mife_get");
+                ex = EX_IOERR;
+                break;
+            }
+            if  (l) s = *cont & 0xFF; else s = -1;
+            p = (u_char)(class[s + 1]);
+            control = automa[state][p];
+            ifBLIN_QV5(exenv->options->flags)
+                BLIN_STATEBODY( stderr
+                              , pn
+                              , n & 0xFF
+                              , clasnames[(u_char)p]
+                              , 0
+                              , (u_char*)cont
+                              , rs
+                              , control
+                              , statenames
+                              , state
+                              , C_state
+                              , (((u_char*)cont)[BLIN_STATEVAR_i]>=' ')
+                              )
+            if  (control & Ct) {
+                if  (++n < max || !max) fputc('\t', fi);
+            }
+            if  (control & Cd) {
+                for (;rs && (n <= max || !max); rs--) fputc(' ', fi);
+            }
+            if  (control & Cb) rs = 0;
+            if  (control & Cp) {
+                if  (n <= max || !max) switch (s) {
+                    case '\\': fprintf(fi, "\\\\");      break;
+                    case '\0': fprintf(fi, "\\0");       break;
+                    case '\t': fprintf(fi, "\\t");       break;
+                    case '\n': fprintf(fi, "\\n");       break;
+                    default  : fputc(s, fi);
+            }   }
+            if  (control & Cn) {
+                for (;++n < max;) {
+                    fprintf(fi, "\t\\N");
+                }
+                fprintf(fi, "\n");
+                n = 0;
+            }
+            if  (control & Cq) rs++;
+            if  (control & Cx) {
+                ifBLIN_QV1(exenv->options->flags) warnx("getcsv syntax");
+                errno = EINVAL;
+                ex = EX_DATAERR;
+        }   }
+        if  (~exenv->options->flags & PGOBLIN_COPEND_NOGEN) fprintf(fi, "\\.\n");
+        if  (fclose(fi) < 0) {
+            ifBLIN_QV1(exenv->options->flags) warn("getcsv fclose on %d", r[PGO_COUT].r);
+            ex = EX_OSERR;
+    }   }
+out:
+    ifBLIN_QV4(exenv->options->flags) fprintf(stderr, "-pgoblin_getcsv %d\n", ex);
+    return(ex);
+}
+
 int
 pgoblin_getdir(pgoblin_exenv *exenv, pgoblin_r *r) {
     int ex = EX_OK, i = 0, max, n;
@@ -447,7 +633,7 @@ pgoblin_getdir(pgoblin_exenv *exenv, pgoblin_r *r) {
             q = NULL;
         }
         if  (++i < max) goto in;
-        fprintf(fi, "\\.\n");
+        if  (~exenv->options->flags & PGOBLIN_COPEND_NOGEN) fprintf(fi, "\\.\n");
     }
 out:
     if  (fi && fclose(fi) < 0) {
