@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2002 Dec 17
+ * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2002 Dec 17\n\
+@(#)Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: cmd_select.c,v 1.42 2009/11/17 07:04:23 babolo Exp $";
+static const char rcsid[] = "$Id: cmd_select.c,v 1.46 2013/03/02 06:23:46 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -39,10 +39,10 @@ static const char rcsid[] = "$Id: cmd_select.c,v 1.42 2009/11/17 07:04:23 babolo
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -86,7 +86,7 @@ pgoblin_perform(pgoblin_exenv *exenv, pgoblin_r *r) {
         if  (DBASE(R_CONN->flags)->flags & PGOBLIN_DB_PREPARE) {
             if  ((ex = DBASE(R_CONN->flags)->prepare(R_CONN, &n, IO_CTL->text))) {
                 ifBLIN_QV1(exenv->options->flags) {
-                    char *c = DBASE(R_CONN->flags)->subinfo(PGOBLIN_ResMess, res, R_CONN->flags);
+                    char *c = DBASE(R_CONN->flags)->subinfo(PGOBLIN_ResMess, n, R_CONN->flags);
 
                     warnx("perform: prepare\n%s", c);
                     DBASE(R_CONN->flags)->finish(R_CONN->flags, PGOBLIN_FrMess, (void**)&c);
@@ -331,7 +331,7 @@ out:
                , (cond == PGOBLIN_CR_SSEL)
                ? "pgoblin_strsel"
                : (cond == PGOBLIN_CR_BSEL) ? "pgoblin_binsel" : "pgoblin_???sel"
-               , (e < 0) ? e : ex
+               , (int)((e < 0) ? e : ex)
                );
     return((e < 0) ? e : ex);
 }
