@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2002 Dec 04
+ * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
+@(#)Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: cmd_trans.c,v 1.55 2009/11/03 21:32:28 babolo Exp $";
+static const char rcsid[] = "$Id: cmd_trans.c,v 1.58 2013/02/24 14:58:48 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -47,11 +47,11 @@ static const char rcsid[] = "$Id: cmd_trans.c,v 1.55 2009/11/03 21:32:28 babolo
 #include <stdio.h>
 #include <errno.h>
 #include <poll.h>
-#include <mife.h>
 #include <err.h>
-#include <multilar.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <multilar.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -149,9 +149,21 @@ pgoblin_exec(pgoblin_exenv *exenv, pgoblin_r *r) {
             argv[i++] = DBASE(IO_CTL->flagc)->getvalue(IO_CTL->pq, exenv->sequen, c);
     }   }
     if  (i) {
+        int eex;
         argv[i] = NULL;
-        ex = JOBE(R_JOB->flags)->exec(exenv, r, argv);
-    }
+        eex = JOBE(R_JOB->flags)->exec(exenv, r, argv);
+        if  (eex) {
+            ifBLIN_QV1(exenv->options->flags) {
+                int z;
+
+                warn("Exec error on %d", r[PGO_CJOB].r);
+                for (z = 0; argv[z]; z++) fprintf(stderr, "[%d]=%s~\n", z, argv[z]);
+            }
+            if  (eex < 0) {
+                if  (eex  < ex) ex = eex;
+            } else if (ex >= 0) {
+                if  (eex > ex) ex = eex;
+    }   }   }
 out:
     ifBLIN_QV4(exenv->options->flags) fprintf( stderr, "-pgoblin_exec %d\n", ex);
     return(ex);
