@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2008 Feb 10
+ * Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2008 Feb 10\n\
+@(#)Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: db_pgsql.c,v 1.27 2009/11/17 07:04:23 babolo Exp $";
+static const char rcsid[] = "$Id: db_pgsql.c,v 1.30 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -38,11 +38,11 @@ static const char rcsid[] = "$Id: db_pgsql.c,v 1.27 2009/11/17 07:04:23 babolo E
 #include <string.h>
 #include <errno.h>
 #include <poll.h>
-#include <mife.h>
 #include <err.h>
 #include <libpq-fe.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -605,8 +605,11 @@ p3info(int kind, void *resv, int row, int col) {
 static char *
 getvalue(void *resv, int row, int col) {
     PGresult *res = ((pgoblin_res *)resv)->res;
-
-    return(PQgetvalue(res, row, col));
+    char *c = NULL;
+    if  (!p3info(PGOBLIN_IsNull, resv, row, col)) {
+        c = PQgetvalue(res, row, col);
+    }
+    return(c);
 }
 
 static int
