@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2008 Apr 14
+ * Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2008 Apr 14\n\
+@(#)Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: db_sqlite2.c,v 1.33 2009/11/04 19:09:23 babolo Exp $";
+static const char rcsid[] = "$Id: db_sqlite2.c,v 1.37 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -40,13 +40,13 @@ static const char rcsid[] = "$Id: db_sqlite2.c,v 1.33 2009/11/04 19:09:23 babolo
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
 #include <sqlite.h>
-#include <multilar.h>
-#include <getCGIparm.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <getCGIparm.h>
+#include <multilar.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -235,7 +235,7 @@ __pgoblin_sqlite2_regexp(sqlite_func *ctx, int argc, const char **argv) {
     if  (!argv[0] || !argv[1]) {
         /* NULL */
     } else {
-        if  (!(preg = malloc(sizeof(preg)))) {
+        if  (!(preg = malloc(sizeof(regex_t)))) {
             sqlite_set_result_error(ctx, "No mem", -1);
             exx = -2147203043;
         } else if ((ex = regcomp(preg, argv[0], REG_EXTENDED | REG_NOSUB))) {
@@ -313,23 +313,23 @@ static const char *statenames[] =
 };
 
 typedef struct pgoblin_db_conn {
-    sqlite     *odf;
-    sqlite_vm  *vm;
-    const char *query;
-    u_int32_t   flags;
-#   define      PGOBLIN_STREAMI 0x000001
-#   define      PGOBLIN_STREAMT 0x000002
-#   define      PGOBLIN_STREAME 0x000004
-#   define      PGOBLIN_ALLOUT  (PGOBLIN_STREAMI | PGOBLIN_STREAMT | PGOBLIN_STREAME)
-#   define      PGOBLIN_COPYIN  0x000010
-#   define      PGOBLIN_COPYST  0x000020
-#   define      PGOBLIN_COPYEN  0x000040
-#   define      PGOBLIN_ALLIN   (PGOBLIN_COPYIN | PGOBLIN_COPYST | PGOBLIN_COPYEN)
-    u_char     *semi;
-    int         ncol;
-    int         coln;
-    u_char      reg;
-    enum states state;
+    sqlite      *odf;
+    sqlite_vm   *vm;
+    const char  *query;
+    u_int32_t    flags;
+#   define       PGOBLIN_STREAMI 0x000001
+#   define       PGOBLIN_STREAMT 0x000002
+#   define       PGOBLIN_STREAME 0x000004
+#   define       PGOBLIN_ALLOUT  (PGOBLIN_STREAMI | PGOBLIN_STREAMT | PGOBLIN_STREAME)
+#   define       PGOBLIN_COPYIN  0x000010
+#   define       PGOBLIN_COPYST  0x000020
+#   define       PGOBLIN_COPYEN  0x000040
+#   define       PGOBLIN_ALLIN   (PGOBLIN_COPYIN | PGOBLIN_COPYST | PGOBLIN_COPYEN)
+    u_char      *semi;
+    int          ncol;
+    int          coln;
+    u_char       reg;
+    enum states  state;
 } pgoblin_db_conn;
 
 typedef struct pgoblin_db_data {
@@ -390,10 +390,10 @@ static const char *
 exdecode(int ex) {
     const char *c;
     switch (ex) {
-    case SQLITE_OK  : c = "SQLITE_OK";         break;
-    case SQLITE_ROW : c = "SQLITE_ROW";        break;
-    case SQLITE_DONE: c = "SQLITE_DONE";       break;
-    default         : c = sqlite_error_string(ex);
+    case SQLITE_OK                     : c = "SQLITE_OK";                      break;
+    case SQLITE_ROW                    : c = "SQLITE_ROW";                     break;
+    case SQLITE_DONE                   : c = "SQLITE_DONE";                    break;
+    default                            : c = sqlite_error_string(ex);
     }
     return(c);
 }
@@ -436,21 +436,22 @@ shurecon(pgoblin_conn *db) {
                                 , "regexp"
                                 , 2
                                 , pgoblin_sqlite2_regexp
-                                , (void*)db->flags
+                                , (void*)(long int)db->flags
                                 );
     ERRMSX("regexp creation", EX_DATAERR);
     ex = sqlite_create_function( conn->odf
                                 , "quote_literal"
                                 , 1
                                 , pgoblin_sqlite2_quote
-                                , (void*)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_SQ)
+                                , (void*)(long int)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_SQ)
                                 );
     ERRMSX("quote_literal creation", EX_DATAERR);
     ex = sqlite_create_function( conn->odf
                                 , "quote_ident"
                                 , 1
                                 , pgoblin_sqlite2_quote
-                                , (void*)( (db->flags & BLIN_VERMASK)
+                                , (void*)(long int)
+                                         ( (db->flags & BLIN_VERMASK)
                                          | PGOBLIN_SQLITE_DQ | PGOBLIN_SQLITE_SF | PGOBLIN_SQLITE_KY
                                 )        );
     ERRMSX("quote_ident creation", EX_DATAERR);
@@ -458,21 +459,21 @@ shurecon(pgoblin_conn *db) {
                                 , "quote_ident_always"
                                 , 1
                                 , pgoblin_sqlite2_quote
-                                , (void*)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_DQ)
+                                , (void*)(long int)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_DQ)
                                 );
     ERRMSX("quote_ident_always creation", EX_DATAERR);
     ex = sqlite_create_function( conn->odf
                                 , "quote_html"
                                 , 1
                                 , pgoblin_sqlite2_quote_html
-                                , (void*)(db->flags & BLIN_VERMASK)
+                                , (void*)(long int)(db->flags & BLIN_VERMASK)
                                 );
     ERRMSX("quote_html creation", EX_DATAERR);
     ex = sqlite_create_function( conn->odf
                                 , "gettimeofday"
                                 , 0
                                 , pgoblin_sqlite2_gettimeofday
-                                , (void*)(db->flags & BLIN_VERMASK)
+                                , (void*)(long int)(db->flags & BLIN_VERMASK)
                                 );
     ERRMSX("gettimeofday creation", EX_DATAERR);
     ifBLIN_QV5(db->flags) {
@@ -602,7 +603,7 @@ putcopy(pgoblin_conn *db, const void *buf, int nby) {
         if  (q < b + nby) control = automa[conn->state][class[*q] >> 4];
           else control = automa[conn->state][Lx >> 4];
         ifBLIN_QV5(db->flags)
-            BLIN_STATEBODY(stderr, pn, conn->reg, clasnames[class[*q]], q, 0, BLIN_I(q), control, statenames, conn->state, C_state, (q[BLIN_STATEVAR_i]!='\n'))
+            BLIN_STATEBODY(stderr, pn, conn->reg, clasnames[class[*q]], q, (int)p - (int)c, BLIN_I(q), control, statenames, conn->state, C_state, (q[BLIN_STATEVAR_i]!='\n'))
         if  (control & Co) c = p = q;
         if  (control & Cb) conn->reg = class[*q] & 0x07;
         if  (control & Cr) conn->reg = (conn->reg << 3) | (class[*q] & 0x07);
@@ -796,6 +797,9 @@ doquery(pgoblin_conn *db, void **rsv, const char *query, u_char cond) {
             ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#1 %d=%s~\n", cond, conn->query);
             ex = sqlite_compile(conn->odf, conn->query, &conn->query, &conn->vm, &errmsg);
             ERRMSX("sqlite_compile#1", EX_SOFTWARE);
+            if  (!conn->vm) {
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#1 no VM\n");
+            }
             conn->flags |= PGOBLIN_COPYIN;
             break;
         case 1:
@@ -939,7 +943,10 @@ doquery(pgoblin_conn *db, void **rsv, const char *query, u_char cond) {
             ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#4 %d=%s~\n", cond, conn->query);
             ex = sqlite_compile(conn->odf, conn->query, &conn->query, &conn->vm, &errmsg);
             ERRMSX("sqlite_compile#4", EX_SOFTWARE);
-            if  (!conn->vm) break;
+            if  (!conn->vm) {
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#4 no VM\n");
+                break;
+            }
             for (;;) {
                 ex = sqlite_step(conn->vm, &n, NULL, NULL);
                 ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_step %d %s\n", ex, exdecode(ex));
@@ -972,7 +979,10 @@ doquery(pgoblin_conn *db, void **rsv, const char *query, u_char cond) {
             ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#5 %d=%s~\n", cond, conn->query);
             ex = sqlite_compile(conn->odf, conn->query, &conn->query, &conn->vm, &errmsg);
             ERRMSX("sqlite_compile#5", EX_SOFTWARE);
-            if  (!conn->vm) break;
+            if  (!conn->vm) {
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_compile#5 no VM\n");
+                break;
+            }
             for (;;) {
                 ex = sqlite_step(conn->vm, &n, &row, NULL);
                 ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite_step %d %s\n", ex, exdecode(ex));
@@ -1304,10 +1314,10 @@ getcopy(pgoblin_conn *db, void **buf) {
         int i, l;
         u_char *c;
 
-        for (i = 0, l = 0; i < n; i++) {
-            if  (row[i]) l += strlen(row[i]);
+        for (i = 0, l = 2; i < n; i++) {
+            if (row[i]) l += strlen(row[i]) * 2 + 1; else l += 3;
         }
-        *buf = malloc(l * 2 + 1);
+        *buf = malloc(l);
         for (i = 0; i < n; i++) {
             if  ((c = (u_char*)row[i])) {
                 if  (getCGIparmpass3(0, (u_char*)"copy", &c)) {
