@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2008 May 10
+ * Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2008 May 10\n\
+@(#)Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: db_sqlite3.c,v 1.20 2009/11/04 19:09:23 babolo Exp $";
+static const char rcsid[] = "$Id: db_sqlite3.c,v 1.24 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -40,13 +40,13 @@ static const char rcsid[] = "$Id: db_sqlite3.c,v 1.20 2009/11/04 19:09:23 babolo
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
 #include <sqlite3.h>
-#include <multilar.h>
-#include <getCGIparm.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <getCGIparm.h>
+#include <multilar.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -239,7 +239,7 @@ __pgoblin_sqlite3_regexp(sqlite3_context *ctx, int argc, sqlite3_value **argv) {
         sqlite3_result_null(ctx);
     } else {
         if  (!(preg = sqlite3_get_auxdata(ctx, 0))) {
-            if  (!(preg = malloc(sizeof(preg)))) {
+            if  (!(preg = malloc(sizeof(regex_t)))) {
                 sqlite3_result_error_nomem(ctx);
                 exx = -2147203043;
             } else if ((ex = regcomp(preg, re, REG_EXTENDED | REG_NOSUB))) {
@@ -506,7 +506,7 @@ shurecon(pgoblin_conn *db) {
         fprintf(stderr, "lib version=%s\n", sqlite3_version);
     }
     if  (!(conn = malloc(sizeof(pgoblin_db_conn)))) {
-        ifBLIN_QV1(db->flags) warnx("shurecon: no RAM");
+        ifBLIN_QV1(db->flags) warnx("sqlite3 shurecon: no RAM");
         ERROUT(EX_OSERR, ENOMEM);
     }
     bzero(conn, sizeof(pgoblin_db_conn));
@@ -516,7 +516,7 @@ shurecon(pgoblin_conn *db) {
                         , NULL
                         );
     if  (!conn->odf) {
-        ifBLIN_QV1(db->flags) warnx("shurecon aborted");
+        ifBLIN_QV1(db->flags) warnx("sqlite3 shurecon aborted");
         ERROUT(EX_UNAVAILABLE, ENODEV);
     }
     ERRMSG("sqlite3_open", EX_UNAVAILABLE);
@@ -530,7 +530,7 @@ shurecon(pgoblin_conn *db) {
                                 , "regexp"
                                 , 2
                                 , SQLITE_ANY
-                                , (void*)db->flags
+                                , (void*)(long int)db->flags
                                 , pgoblin_sqlite3_regexp
                                 , NULL
                                 , NULL
@@ -540,7 +540,7 @@ shurecon(pgoblin_conn *db) {
                                 , "quote_literal"
                                 , 1
                                 , SQLITE_ANY
-                                , (void*)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_SQ)
+                                , (void*)(long int)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_SQ)
                                 , pgoblin_sqlite3_quote
                                 , NULL
                                 , NULL
@@ -550,7 +550,8 @@ shurecon(pgoblin_conn *db) {
                                 , "quote_ident"
                                 , 1
                                 , SQLITE_ANY
-                                , (void*)( (db->flags & BLIN_VERMASK)
+                                , (void*)(long int)
+                                         ( (db->flags & BLIN_VERMASK)
                                          | PGOBLIN_SQLITE_DQ | PGOBLIN_SQLITE_SF | PGOBLIN_SQLITE_KY
                                          )
                                 , pgoblin_sqlite3_quote
@@ -562,7 +563,7 @@ shurecon(pgoblin_conn *db) {
                                 , "quote_ident_always"
                                 , 1
                                 , SQLITE_ANY
-                                , (void*)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_DQ)
+                                , (void*)(long int)((db->flags & BLIN_VERMASK) | PGOBLIN_SQLITE_DQ)
                                 , pgoblin_sqlite3_quote
                                 , NULL
                                 , NULL
@@ -572,7 +573,7 @@ shurecon(pgoblin_conn *db) {
                                 , "quote_html"
                                 , 1
                                 , SQLITE_ANY
-                                , (void*)(db->flags & BLIN_VERMASK)
+                                , (void*)(long int)(db->flags & BLIN_VERMASK)
                                 , pgoblin_sqlite3_quote_html
                                 , NULL
                                 , NULL
@@ -582,7 +583,7 @@ shurecon(pgoblin_conn *db) {
                                 , "gettimeofday"
                                 , 0
                                 , SQLITE_ANY
-                                , (void*)(db->flags & BLIN_VERMASK)
+                                , (void*)(long int)(db->flags & BLIN_VERMASK)
                                 , pgoblin_sqlite3_gettimeofday
                                 , NULL
                                 , NULL
@@ -825,6 +826,7 @@ finish(u_int32_t flags, int kind, void **entity) {
 
             if  (vm && *vm) {
                 ex = sqlite3_finalize(*vm);
+                ifBLIN_QV5(flags) fprintf(stderr, "sqlite3_finalize %d %s\n", ex, exdecode(ex));
                 if  ((ex != SQLITE_OK) && (ex != SQLITE_ROW) && (ex != SQLITE_DONE)) {
                     ifBLIN_QV1(flags) warnx("sqlite3_finalize %d: %s", ex, exdecode(ex));
                     ex = EX_SOFTWARE;
@@ -1048,7 +1050,10 @@ doquery(pgoblin_conn *db, void **rsv, const char *query, u_char cond) {
             ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite3_prepare_v2#4 %d=%s~\n", cond, conn->query);
             ex = sqlite3_prepare_v2(conn->odf, conn->query, -1, &conn->vm, &conn->query);
             ERRMSG("sqlite3_prepare_v2#4", EX_SOFTWARE);
-            if  (!conn->vm) break;
+            if  (!conn->vm) {
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite3_prepare_v2#4 no VM\n");
+                break;
+            }
             n = sqlite3_column_count(conn->vm);
             if  (n) {
                 ifBLIN_QV2(db->flags) warnx("wait for PGOBLIN_CR_OK but received %d cols", n);
@@ -1082,11 +1087,15 @@ doquery(pgoblin_conn *db, void **rsv, const char *query, u_char cond) {
             ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite3_prepare_v2#5 %d=%s~\n", cond, conn->query);
             ex = sqlite3_prepare_v2(conn->odf, conn->query, -1, &conn->vm, &conn->query);
             ERRMSG("sqlite3_prepare_v2#5", EX_SOFTWARE);
-            if  (!conn->vm) break;
+            if  (!conn->vm) {
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite3_prepare_v2#5 no VM\n");
+                break;
+            }
             res->ncol = sqlite3_column_count(conn->vm);
             for (;;) {
                 ex = sqlite3_step(conn->vm);
                 ERRMSG("sqlite3_step#3", EX_SOFTWARE);
+                ifBLIN_QV5(db->flags) fprintf(stderr, "sqlite3_step %d %s\n", ex, exdecode(ex));
                 if  (!(res->flags & PGOBLIN_TINIT)) {
                     res->nrow = 0;
                     res->flags |= PGOBLIN_TINIT;
@@ -1415,8 +1424,10 @@ getcopy(pgoblin_conn *db, void **buf) {
         int i, l;
         u_char *c;
 
-        for (i = 0, l = 0; i < n; i++) if (row[i]) l += strlen((char*)row[i]);
-        *buf = malloc(l * 2 + 1);
+        for (i = 0, l = 2; i < n; i++) {
+            if (row[i]) l += strlen((char*)row[i]) * 2 + 1; else l += 3;
+        }
+        *buf = malloc(l);
         for (i = 0; i < n; i++) {
             if  ((c = row[i])) {
                 if  (getCGIparmpass3(0, (u_char*)"copy", &c)) {
