@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2007 Nov 20
+ * Copyright (C)2007..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,16 +26,17 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2007 Nov 20\n\
+@(#)Copyright (C)2007..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: descript.c,v 1.40 2009/11/17 07:04:23 babolo Exp $";
+static const char rcsid[] = "$Id: descript.c,v 1.44 2013/02/05 06:14:37 babolo Exp $";
 #endif /* not lint */
 
 #define _PGOBLIN_CHAR_CLASSES_ 1
 #include <sys/types.h>
 #include <stdio.h>
-#include <mife.h>
+#include <babolo/BLINflag.h>
 #include <babolo/parser.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 #include "cmds.h"
@@ -313,7 +314,16 @@ const pgoblin_syntax pgoblin =
 // 0x08 mode 
 // 0x10 uid 
 // 0x20 gid 
-  , { Fx                      , {    0,     0,     0,    0}, NULL           , NULL}        /* 028 */
+  , {    Io|Id|Ii             , {Po|Pm,    Pt,     0,   Pm}, pgoblin_getcsv , "getcsv"}    /* 028 */
+//  CSV   COPY .
+//  CTL    2      ,
+//      ,
+//         ,
+//     NULL  .
+//    ,       
+// ,      .
+//  CTL ,      ';',
+//     '"'.
   , { Fx                      , {    0,     0,     0,    0}, NULL           , NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           , NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           , NULL}
