@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2002 Dec 23
+ * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2002 Dec 23\n\
+@(#)Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: execute.c,v 1.80 2009/11/17 09:04:40 babolo Exp $";
+static const char rcsid[] = "$Id: execute.c,v 1.86 2013/03/02 06:23:46 babolo Exp $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -43,11 +43,11 @@ static const char rcsid[] = "$Id: execute.c,v 1.80 2009/11/17 09:04:40 babolo Ex
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
-#include <multilar.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <multilar.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -168,7 +168,7 @@ pgoblin_fini(pgoblin_main *options) {
 
 static void
 dump_ioreg(pgoblin_main *options, int r) {
-    u_int32_t wdr, cdr;
+    void *wdr, *cdr;
 
     fprintf( stderr, "[%c](%06X%s%s%s%s%s%s%s%s%s%s%s%s):"
            , pgoblin_regn[r], R.flags
@@ -193,14 +193,14 @@ dump_ioreg(pgoblin_main *options, int r) {
         fprintf(stderr, " pq=%" BLIN_X, BLIN_I(R.pq));
         fprintf(stderr, " flagc=%08X", R.flagc);
     }
-    wdr = BLIN_I(R.wri);
-    if  (wdr == BLIN_I(write)) fprintf(stderr, " wri=write");
-    else if (wdr == BLIN_I(mife_writ)) fprintf(stderr, " wri=mife_writ");
-    else if (wdr) fprintf(stderr, " wri=%" BLIN_X, wdr);
+    wdr = R.wri;
+    if  (wdr == write) fprintf(stderr, " wri=write");
+    else if (wdr == mife_writ) fprintf(stderr, " wri=mife_writ");
+    else if (wdr) fprintf(stderr, " wri=%" BLIN_X, BLIN_I(wdr));
 
-    cdr = BLIN_I(R.clo);
-    if  (cdr == BLIN_I(close)) fprintf(stderr, " clo=close");
-    else if (cdr) fprintf(stderr, " clo=%08X", cdr);
+    cdr = R.clo;
+    if  (cdr == close) fprintf(stderr, " clo=close");
+    else if (cdr) fprintf(stderr, " clo=%" BLIN_X, BLIN_I(cdr));
 
     if  (wdr || cdr) fprintf(stderr, " onu=%d", R.onu);
     fprintf(stderr, "\n");
@@ -717,55 +717,6 @@ out:
     return(ex);
 }
 
-static int
-markproc(pgoblin_jobreg *g, int i, int nq, struct kevent *pkev) {
-    int ex = EX_OK, exx = EX_OK, status;
-    pgoblin_jobl *j;
-
-    if  (nq == 0) {
-    } else if ((nq > 0) && (pkev->filter == EVFILT_PROC) && (pkev->fflags & NOTE_EXIT)) {
-        int i;
-
-        for (i = 0; i < MULAR_NEXT(g->pids); i++) {
-            j = mular_getix(g->pids, i);
-            if  ((j->pid == pkev->ident) && (j->cmdn >= 0) && (j->status == PGOBLIN_NOSTATUS)) {
-                if  (!(ex = waitpid(j->pid, &status, WNOHANG))) {
-                    ifBLIN_QV1(g->flags) warnx("No status for pid %d at %d", j->pid, (int)j->cmdn);
-                } else if (ex == j->pid) {
-                    j->status = status;
-                    g->dead++;
-                    ifBLIN_QV4(g->flags)
-                        fprintf( stderr
-                               , "markproc st=%08X fl=%08X ex=%d exx=%d IFEX=%d EXST=%d\n"
-                               , status, g->flags, ex, exx, WIFEXITED(status), WEXITSTATUS(status)
-                               );
-                    if  (  (g->flags & PGOBLIN_ERROR_SENSOR)
-                        && WIFEXITED(status)
-                        && WEXITSTATUS(status)
-                        ) exx = WEXITSTATUS(status);
-                    if  ((g->flags & PGOBLIN_ERROR_SENSIB) && exx) break;
-                } else if (ex > 0) {
-                    ifBLIN_QV1(g->flags) warn("Pid %d<>%d at %d", ex, j->pid, (int)j->cmdn);
-                } else if (errno == ECHILD || errno == EDEADLK || errno == EAGAIN) {
-                } else {
-                    ifBLIN_QV1(g->flags) warn("Pid %d in %d", j->pid, (int)j->cmdn);
-                }
-                ex = EX_OK;
-                goto out;
-        }   }
-        ifBLIN_QV1(g->flags) warnx("Pid %d not found", (int)pkev->ident);
-    } else if ((errno == ESRCH) && (i >= 0)) {
-        j = mular_getix(g->pids, i);
-        j->status = PGOBLIN_LOSTSTAT;
-        g->dead++;
-    } else if (errno != EINTR) {
-        ifBLIN_QV1(g->flags) warn("kevent for pid %d", (int)pkev->ident);
-        ex = EX_OSERR;
-    }
-out:
-    return(ex ? ex : exx);
-}
-
 static int
 /**********************************************************************
  **********************************************************************
@@ -774,39 +725,77 @@ waitjob(pgoblin_jobreg *g) {                                        /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-    int ex = EX_OK, i, nq;
-    struct timespec timeout;
-    struct kevent kev;
+    int ex = EX_OK, exx = EX_OK, i, status;
     pgoblin_jobl *j;
+    struct kevent kev;
 
-    bzero(&timeout, sizeof(timeout));
     bzero(&kev, sizeof(kev));
-    if  (!(g->flags & PGOBLIN_KQUEUED)) {
-        if  ((g->kq = kqueue()) < 0) {
-            ex = EX_OSERR;
-            goto out;
-        } else g->flags |= PGOBLIN_KQUEUED;
+    EV_SET(&kev, ex, EVFILT_PROC, EV_ADD, NOTE_EXIT, 0, NULL);
+    if  (!(g->flags & PGOBLIN_KQUEUED) && ((g->kq = kqueue()) < 0)) {
+        ex = EX_OSERR;
+        goto out;
     }
-    for (i = 0; i < MULAR_NEXT(g->pids); i++) {
+    MARK_R_JOB_GO(g);;
+    ;;  g->flags |= PGOBLIN_KQUEUED;
+    ;;  if  (kevent(g->kq, &kev, 1, NULL, 0, NULL)) {
+    ;;      ex = EX_OSERR;
+    ;;      goto out;
+    ;;  }
+    MARK_R_JOB_WENT(g);;
+try:
+    for (i = 0; !ex && (i < MULAR_NEXT(g->pids)); i++) {
         j = mular_getix(g->pids, i);
         if  (j->pid <= 0) {
             ifBLIN_QV1(g->flags) warnx("No pid in %d", (int)j->cmdn);
             errno = EINVAL;
             ex = EX_SOFTWARE;
-        } else if ((j->cmdn >= 0) && (j->status == PGOBLIN_NOSTATUS)) {
+        } else if (j->cmdn < 0) {
+            ifBLIN_QV1(g->flags) warnx("No cmdn %d", (int)j->cmdn);
+            errno = EINVAL;
+            ex = EX_SOFTWARE;
+        } else if (j->status == PGOBLIN_NOSTATUS) {
             EV_SET(&kev, j->pid, EVFILT_PROC, EV_ADD | EV_ONESHOT | EV_CLEAR, NOTE_EXIT, 0, NULL);
-            nq = kevent(g->kq, &kev, 1, &kev, 1, &timeout);
-            ex = markproc(g, i, nq, &kev);
-            if  (ex) goto out;
-    }   }
-    nq = 0;
-    if  (MULAR_NEXT(g->pids) >= (g->concur + g->dead)) nq = kevent(g->kq, NULL, 0, &kev, 1, NULL);
-    ex = markproc(g, -1, nq, &kev);
-    while ((nq = kevent(g->kq, NULL, 0, &kev, 1, &timeout)) && !ex) {
-        ex = markproc(g, -1, nq, &kev);
+            if  (0 > kevent(g->kq, &kev, 1, NULL, 0, NULL)) {
+                ifBLIN_QV1(g->flags) warn("No pid %d for kevent ADD", (int)j->pid);
+                ex = EX_SOFTWARE;
+            } else if (0 > (ex = waitpid(j->pid, &status, WNOHANG))) {
+                if  (errno == ECHILD) {
+                    ifBLIN_QV2(g->flags) warnx("Lost pid %d in %d", j->pid, (int)j->cmdn);
+                    MARK_R_JOB_GO(g);;
+                    ;;  j->status = PGOBLIN_LOSTSTAT;
+                    ;;  g->dead++;
+                    MARK_R_JOB_WENT(g);;
+                    errno = 0;
+                    ex = 0;
+                } else {
+                    ifBLIN_QV1(g->flags) warn("Pid %d in %d", j->pid, (int)j->cmdn);
+                }
+            } else if (0 < ex) {
+                if  (ex == j->pid) {
+                    EV_SET(&kev, j->pid, EVFILT_PROC, EV_DELETE, NOTE_EXIT, 0, NULL);
+                    if  (0 > kevent(g->kq, &kev, 1, NULL, 0, NULL)) {
+                        ifBLIN_QV2(g->flags) warn("No pid %d for kevent DELETE", (int)j->pid);
+                    }
+                    MARK_R_JOB_GO(g);;
+                    ;;  j->status = status;
+                    ;;  g->dead++;
+                    MARK_R_JOB_WENT(g);;
+                    ex = 0;
+                    if  (  (g->flags & PGOBLIN_ERROR_SENSOR)
+                        && WIFEXITED(status)
+                        && WEXITSTATUS(status)
+                        && !exx
+                        ) exx = WEXITSTATUS(status);
+                    if  ((g->flags & PGOBLIN_ERROR_SENSIB) && exx) break;
+                } else {
+                    ifBLIN_QV1(g->flags) warnx("Pid %d<>%d at %d", ex, j->pid, (int)j->cmdn);
+    }   }   }   }
+    if  (!ex && g->concur && (MULAR_NEXT(g->pids) >= (g->concur + g->dead))) {
+        kevent(g->kq, NULL, 0, &kev, 1, NULL);
+        goto try;
     }
 out:
-    return(ex);
+    return(ex ? ex : exx);
 }
 
 static int
@@ -1182,6 +1171,7 @@ pgoblin_execute( pgoblin_main *options       /**/
     ;               ;;  R_JOB->host = NULL;
     ;               ;;  R_JOB->concur = 0;
     ;               ;;  R_JOB->dead = 0;
+    ;               ;;  if  (R_JOB->flags & PGOBLIN_KQUEUED) close(R_JOB->kq);
     ;               ;;  R_JOB->flags = pgm->flags & BLIN_VERMASK;
     ;               MARK_R_JOB_WENT(R_JOB);;
     ;               if  (ex) {
@@ -1202,7 +1192,7 @@ pgoblin_execute( pgoblin_main *options       /**/
     ;       if  (CH_STY & PGOBLIN_OUTSET) {
     ;           if  (r[PGO_CSTY].r) {
     ;               MARK_R_STYLE_GO(R_STYLE);;
-    ;               ;;  if  (R_STYLE->style) STYLS(R_STYLE->flags)->free(R_STYLE);
+    ;               ;;  if  (R_STYLE->style) STYLS(R_STYLE->flags)->clear(R_STYLE);
     ;               ;;  R_STYLE->style = NULL;
     ;               ;;  R_STYLE->path = NULL;
     ;               ;;  R_STYLE->flags = pgm->flags & BLIN_VERMASK;
@@ -1302,10 +1292,12 @@ pgoblin_execute( pgoblin_main *options       /**/
                     pgoblin_jobl *j;
 
     ;       ;       ifBLIN_QV4(pgm->flags) fprintf(stderr, "FORK DONE %d\n", ex);
-    ;       ;       j = mular_add(R_JOB->pids);
-    ;       ;       j->cmdn = exenv->ppoint;
-    ;       ;       j->pid = ex;
-    ;       ;       j->status = PGOBLIN_NOSTATUS;
+    ;       ;       MARK_R_JOB_GO(R_JOB);;
+    ;       ;       ;;  j = mular_add(R_JOB->pids);
+    ;       ;       ;;  j->cmdn = exenv->ppoint;
+    ;       ;       ;;  j->pid = ex;
+    ;       ;       ;;  j->status = PGOBLIN_NOSTATUS;
+    ;       ;       MARK_R_JOB_WENT(R_JOB);;
     ;       ;       if  (cmd == PGOBLIN_fork) exenv->lexlevel++;
     ;       ;       ex = 0;
     ;   }   }   }
@@ -1474,7 +1466,7 @@ pgoblin_execute( pgoblin_main *options       /**/
 
 #           define O(G) (exenv->options->style[G])
             MARK_R_STYLE_GO(&O(g));;
-            ;;  STYLS(O(g).flags)->free(&O(g));
+            ;;  STYLS(O(g).flags)->clear(&O(g));
             ;;  O(g).style = NULL;
             ;;  O(g).path = NULL;
             ;;  O(g).flags &= ~PGOBLIN_STYLE_TYPE;
