@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2002 Dec 04
+ * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
+@(#)Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: pgoblin.c,v 1.46 2009/11/05 01:15:33 babolo Exp $";
+static const char rcsid[] = "$Id: pgoblin.c,v 1.51 2013/02/24 14:58:48 babolo Exp $";
 #endif /* not lint */
 
 #define _PGOBLIN_CHAR_CLASSES_ 1
@@ -40,10 +40,10 @@ static const char rcsid[] = "$Id: pgoblin.c,v 1.46 2009/11/05 01:15:33 babolo Ex
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-#include <mife.h>
 #include <err.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -540,9 +540,9 @@ main(int argc, char *argv[], char **envp) {
     pgm->debug = NULL;
     opt = NULL;
     if  (babolo_openopt( Bpars_NONU, &opt, argc, argv
-                       , flar = "aAB:c:C:d:D:e:Ef:Fh:INOp:qr:R:s:S:t:T:U:vVw:zZ?"
+                       , flar = "aAbB:c:C:d:D:e:Ef:Fh:INOp:qr:R:s:S:t:T:U:vVw:zZ?"
         )              ) {
-        errx(EX_SOFTWARE, "openopt");
+        err(EX_SOFTWARE, "openopt");
     }
     for (redirenv = 0; (c = babolo_getopt(&opt)) != 0; redirenv--) {
         if  (redirenv < 0) redirenv = 0;
@@ -551,6 +551,9 @@ main(int argc, char *argv[], char **envp) {
 //    COPY ( \.)    #copyout.
         case 'A': options->flags |= PGOBLIN_COPEND_APND;                                         break;
 //    COPY ( \.)    #copyout,    .
+        case 'b': options->flags |= PGOBLIN_COPEND_NOGEN;                                        break;
+//     COPY ( \.)  #get***.
+//     #get*** ,   getCGIparm.
         case 'B': if  (dbn >= 0) {
 // Ar 
 //    .
@@ -731,7 +734,7 @@ main(int argc, char *argv[], char **envp) {
     options->sp = 0;
     if  (Tflag) {
         switch (Tflag[0]) {
-        case '0': ex = pgoblin_htman(options->flags, Tflag); break;
+        case '0': ex = pgoblin_htman(options->flags, flar); break;
         case '1': ex = pgoblin_htman(options->flags, Tflag); break;
         case '2': ex = pgoblin_keycodes(options);            break;
         }
