@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2002 Dec 04
+ * Copyright (C)2002..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: pgoblin.h,v 1.79 2009/11/05 01:15:33 babolo Exp $
+ * $Id: pgoblin.h,v 1.82 2011/12/10 04:51:09 babolo Exp $
  */
 
 #ifndef _PGOBLIN_H_
@@ -130,6 +130,7 @@ typedef enum pgoblin_command
 , PGOBLIN_copyout    = 0x29
 , PGOBLIN_getstr     = 0x2A
 , PGOBLIN_getdir     = 0x2B
+, PGOBLIN_getcsv     = 0x2C
 
 , PGOBLIN_begin   = PGOBLIN_EXTENDED | 0x00
 , PGOBLIN_end     = PGOBLIN_EXTENDED | 0x01
@@ -285,6 +286,7 @@ typedef struct pgoblin_main {
 #   define           PGOBLIN_VERIFY       0x000100 /* Verify analizer and dump parsed program        */
 #   define           PGOBLIN_COPEND_PASS  0x000200 /* Pass through unconsumed end of file            */
 #   define           PGOBLIN_COPEND_APND  0x000400 /* Append EOF marker to COPY out file             */
+#   define           PGOBLIN_COPEND_NOGEN 0x000800 /* Not append EOF marker to #get***               */
 /*  define           PGOBLIN_ERROR_SENSOR 0x400000    Eror reaction                                  */
 /*  define           PGOBLIN_ERROR_SENSIB 0x800000    Immediate error reaction                       */
 #   define           PGOBLIN_CLEAR_ONEXIT 0x010000 /* clear registers on exit                        */
@@ -465,7 +467,7 @@ struct pgoblin_styles {
     int    (*fini)(u_int32_t flags, pgoblin_styles *this);
     int  (*parser)(pgoblin_styreg *reg_style, u_char *in);
     void   (*dump)(pgoblin_styreg *reg_style);
-    int    (*free)(pgoblin_styreg *reg_style);
+    int   (*clear)(pgoblin_styreg *reg_style);
     char   *(*get)(pgoblin_styreg *reg_style, u_char *stringname);
     int   (*table)( pgoblin_io    *reg_out
                   , pgoblin_styreg*reg_style
