@@ -1,5 +1,5 @@
 /*-
- * Copyright (C) @BABOLO  2008 Feb 10
+ * Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,9 +26,9 @@
 
 #ifndef lint
 static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2008 Feb 10\n\
+@(#)Copyright (C)2008..2011 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: style_pgo0.c,v 1.32 2009/11/04 19:09:23 babolo Exp $";
+static const char rcsid[] = "$Id: style_pgo0.c,v 1.37 2013/03/02 06:23:46 babolo Exp $";
 #endif /* not lint */
 
 /*        HVUDLR:
@@ -43,11 +43,11 @@ static const char rcsid[] = "$Id: style_pgo0.c,v 1.32 2009/11/04 19:09:23 babolo
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <mife.h>
 #include <err.h>
-#include <multilar.h>
-#include <babolo/parser.h>
 #include <babolo/BLINflag.h>
+#include <babolo/parser.h>
+#include <multilar.h>
+#include <mife.h>
 #include "pgoblin.h"
 #include "pgob.h"
 
@@ -603,9 +603,10 @@ sout(pgoblin_io *reg_out, char *c, int l) {
  **   ,   + .     ,      **
  **     .                                                              **/
 static char *                                                                                      /**/
-dsig( u_int32_t fl, mular_descriptor *s, ssize_t f, ssize_t b, pgo0_ent *pr) {                     /**
+dsig(u_int32_t fl, mular_descriptor *s, ssize_t f, ssize_t b, pgo0_ent *pr) {                      /**
  *****************************************************************************************************/
     ssize_t canf = 0, canb = 0, can=0;
+    char **ss = NULL;
    /******************************************************/
     u_int tmpidx;  /* Due to the error in cc -v          *
     *** Using built-in specs.                          ***
@@ -613,13 +614,13 @@ dsig( u_int32_t fl, mular_descriptor *s, ssize_t f, ssize_t b, pgo0_ent *pr) {
     *** Thread model: posix                            ***
     *** gcc version 3.4.6 [FreeBSD] 20060305           ***
     * I can't use direct                                 *
-    *** mular_getix(st->str, ~can)                     ***
+    *** mular_getix(s, ~can)                           ***
     ******************************************************/
 
-    if  (fl & FIRST) {
+    if  ((fl & FIRST) && (f < (1 << PGOBLIN_NTBSIZE))) {
         canf = pr->spec[f].f;
     }
-    if  (fl & LAST) {
+    if  ((fl & LAST) && (b < (1 << PGOBLIN_NTBSIZE))) {
         canb = pr->spec[b].b;
     }
     if  (!canf) can = canb;
@@ -630,8 +631,13 @@ dsig( u_int32_t fl, mular_descriptor *s, ssize_t f, ssize_t b, pgo0_ent *pr) {
     else can = canf;
     if  (!can) can = pr->def;
     tmpidx = ~can;
-    return((!can) ? NULL : *(char**)mular_getix(s, tmpidx));
-}
+    if  (can) ss = (char**)mular_getix(s, tmpidx);
+    if  (!ss) {
+        ifBLIN_QV2(fl) warnx("style pgo0 tbl NULL @ f=%"BLIN_D" b=%"BLIN_D, f, b);
+        return(NULL);
+    } else {
+        return(*ss);
+}   }
 
 static int
 pgoblin_style_tbl( pgoblin_io    *reg_out
